/****************************************************************************/
/* Error correcting code - Reed Miller RM(1,5)                              */
/* 2014 cermakk.cz															*/
/*																			*/
/* Module for encoding and decoding of given data. It provides encoding of	*/
/* 6bit long words into 32bit long message and vice versa. Corrects up to 7 */
/* "1bit mistakes" in encoded message. 										*/
/*																			*/
/* Functions:																*/
/* ---------------															*/
/* uint32_t encode(uint8_t message) Encode message, returns encoded data.	*/
/* uint8_t decode(uint32_t message, uint8_t* return_vector) decode the	 	*/
/*		given message to return_vector and returns decoding status.			*/
/*																			*/
/****************************************************************************/

#ifndef _ECC_H_
#define _ECC_H_ 1




/************************************************************************/
/*                         Public functions                             */
/************************************************************************/



#include <avr/pgmspace.h>
   
/*
	Warning: max 6 bit input message is allowed (0-3f) !!!!!!

	encode message byte with RM(1,5), returns encoded vector
*/
uint32_t encode(uint8_t message);


/*
	decode and correct message with RM(1,5)
	returns 0=OK; >0 = NOK
	decoded vector is returned through pointer
*/
uint8_t decode(uint32_t message, uint8_t* return_vector);







/************************************************************************/
/*                             EXAMPLES                                 */
/************************************************************************/
/*
	Example: encode data, provide an error to encoded message and decode
			 the message.

	#include "ecc.h"
	.
	.
	.
	int main(void)
	{
		uint8_t decoded_message=0;
		volatile uint32_t encoded_message = 0;
		volatile uint32_t encoded_message_with_failure = 0;
		volatile uint8_t codeword=0;
		
		codeword=0x19; // e.g. 0x19 to be encoded		
		encoded_message = encode(codeword); // encode my codeword into encoded message

		encoded_message_with_failure = encoded_message^=0x05; // insert two "mistakes"

		if(decode(encoded_message_with_failure, &decoded_message)>0)	// decode the message and 
		{
			// error in decoding occurred
		}
		.
		.

		
		// Results: codeword = 0x19
		//			encoded_message = 0xc3c3c3c3
		//			encoded_message_with_failure = 0xc3c3c2c2
		//			decoded_message = 0x19
	}
	

*/





/************************************************************************/
/*                               CODE                                   */
/************************************************************************/

const unsigned long GenMatrix[] PROGMEM={
	0xFFFFFFFF, // 1
	0x0000FFFF, // v5
	0x00FF00FF, // v4
	0x0F0F0F0F, // v3
	0x33333333, // v2
	0x55555555 // v1
};
const unsigned long Reed[] PROGMEM={
	0x00000003,
	0x00030000,
	0x00000300,
	0x03000000,
	0x00000030,
	0x00300000,
	0x00003000,
	0x30000000,
	0x0000000C,
	0x000C0000,
	0x00000C00,
	0x0C000000,
	0x000000C0,
	0x00C00000,
	0x0000C000,
	0xC0000000,
	0x00000005,
	0x00050000,
	0x00000500,
	0x05000000,
	0x00000050,
	0x00500000,
	0x00005000,
	0x50000000,
	0x0000000A,
	0x000A0000,
	0x00000A00,
	0x0A000000,
	0x000000A0,
	0x00A00000,
	0x0000A000,
	0xA0000000,
	0x00000011,
	0x00110000,
	0x00001100,
	0x11000000,
	0x00000044,
	0x00440000,
	0x00004400,
	0x44000000,
	0x00000022,
	0x00220000,
	0x00002200,
	0x22000000,
	0x00000088,
	0x00880000,
	0x00008800,
	0x88000000,
	0x00000101,
	0x01010000,
	0x00001010,
	0x10100000,
	0x00000404,
	0x04040000,
	0x00004040,
	0x40400000,
	0x00000202,
	0x02020000,
	0x00002020,
	0x20200000,
	0x00000808,
	0x08080000,
	0x00008080,
	0x80800000,
	0x00010001,
	0x01000100,
	0x00100010,
	0x10001000,
	0x00040004,
	0x04000400,
	0x00400040,
	0x40004000,
	0x00020002,
	0x02000200,
	0x00200020,
	0x20002000,
	0x00080008,
	0x08000800,
	0x00800080,
	0x80008000
};
volatile unsigned char err=0;
unsigned char scalar_mult(unsigned long _vect)
{
	unsigned char ones=0;
	for(unsigned char index=0;index<32;index++)
	{
		if((_vect&1)>0) ones++;
		_vect>>=1;
	}
	return (ones%2);
}
unsigned char majority(unsigned char* _result_array)
{
	unsigned char ones_cnt=0;
	for (unsigned char i=0;i<16;i++)
	{
		if(*_result_array > 0)
		ones_cnt++;
		_result_array++;
	}
	if(ones_cnt==8) return 2;
	if(ones_cnt>8) return 1;
	return 0;
}
unsigned char majority2(unsigned long _dword)
{
	unsigned char ones_cnt=0;
	for (unsigned char i=0;i<32;i++)
	{
		if((_dword&1) > 0)
		ones_cnt++;
		_dword>>=1;
	}
	if(ones_cnt==16) return 2;
	if(ones_cnt>16) return 1;
	return 0;
}
unsigned long RM_encode(unsigned char message)
{
	unsigned long result=0;
	// 8 bit:
	// |-|-|v5|v4|v3|v2|v1|1|
	if((message&0x01)>0) result^=pgm_read_dword(&GenMatrix[0]);
	if((message&0x02)>0) result^=pgm_read_dword(&GenMatrix[1]);
	if((message&0x04)>0) result^=pgm_read_dword(&GenMatrix[2]);
	if((message&0x08)>0) result^=pgm_read_dword(&GenMatrix[3]);
	if((message&0x10)>0) result^=pgm_read_dword(&GenMatrix[4]);
	if((message&0x20)>0) result^=pgm_read_dword(&GenMatrix[5]);
	return result;
}
unsigned char RM_decode(unsigned long message)
{
	unsigned long tmp=0;
	unsigned char result[16],i;
	unsigned char decoded_bits[6];
	for(i=0;i<16;i++)
	{
		tmp=(pgm_read_dword(&Reed[i]) & message);
		result[i]=scalar_mult(tmp);
	}
	decoded_bits[0]=majority(&result[0]);
	if(decoded_bits[0]>1) err++;
	for(i=0;i<16;i++)
	{
		tmp=(pgm_read_dword(&Reed[i+16]) & message);
		result[i]=scalar_mult(tmp);
	}
	decoded_bits[1]=majority(&result[0]);
	if(decoded_bits[1]>1) err++;
	for(i=0;i<16;i++)
	{
		tmp=(pgm_read_dword(&Reed[i+32]) & message);
		result[i]=scalar_mult(tmp);
	}
	decoded_bits[2]=majority(&result[0]);
	if(decoded_bits[2]>1) err++;
	for(i=0;i<16;i++)
	{
		tmp=(pgm_read_dword(&Reed[i+48]) & message);
		result[i]=scalar_mult(tmp);
	}
	decoded_bits[3]=majority(&result[0]);
	if(decoded_bits[3]>1) err++;
	for(i=0;i<16;i++)
	{
		tmp=(pgm_read_dword(&Reed[i+64]) & message);
		result[i]=scalar_mult(tmp);
	}
	decoded_bits[4]=majority(&result[0]);
	if(decoded_bits[4]>1) err++;
	// decoding of decoded_bit[5] using majority
	tmp=0;
	if((decoded_bits[0])>0) tmp^=pgm_read_dword(&GenMatrix[5]); //x1
	if((decoded_bits[1])>0) tmp^=pgm_read_dword(&GenMatrix[4]); //x2
	if((decoded_bits[2])>0) tmp^=pgm_read_dword(&GenMatrix[3]); //x3
	if((decoded_bits[3])>0) tmp^=pgm_read_dword(&GenMatrix[2]); //x4
	if((decoded_bits[4])>0) tmp^=pgm_read_dword(&GenMatrix[1]); //x5
	tmp^=message;
	decoded_bits[5]=majority2(tmp);
	if(decoded_bits[5]>1) err++;
	return(decoded_bits[0]<<5 | decoded_bits[1]<<4 | decoded_bits[2]<<3 |
	decoded_bits[3]<<2 | decoded_bits[4]<<1 | decoded_bits[5]<<0);
}

uint32_t encode(uint8_t message){
/*
encode message byte with RM(1,5), returns encoded vector
*/
	if (message>0x3f) //maximum 6bit message allowed
	{			
		return 0;
	}
	return(RM_encode(message));

}

uint8_t decode(uint32_t message, uint8_t* return_vector)
{
/*
decode and correct message with RM(1,5)
returns 0=OK; >0 = NOK
decoded vector is returned through pointer
*/
	*return_vector = RM_decode(message);
	return err;

}
#endif /* _ECC_H_ */