/****************************************************************************/
/* Remote Control				                                            */
/* 2014 cermakk.cz															*/
/*																			*/
/* Module deals with the power management and battery measurements.			*/
/*																			*/
/*																			*/
/* Functions:																*/
/* ---------------															*/
/* remote_init(); Initialize this SW module									*/
/* uint8_t remote_recieve_wait(); Wait until data are received, copy all	*/
/*     data into rx buffer and returns first data byte.						*/ 
/* uint8_t remote_recieve_nowait(); Check for data but didn't stop the		*/
/*     program run. Returns 0 if no data are available, >0 if new data are	*/
/*	   stored into rx_buffer.												*/
/* void remote_read_buffer(uint8_t *destination); Copy the rx buffer into	*/
/*	   desired destination.													*/
/****************************************************************************/

// Warning: uses same PINS as SPI, but this is a SW version of SPI!!!!

#ifndef _REMOTE_H_
#define _REMOTE_H_ 1

// Libraries
#include "mcu.h"
#include <string.h>

// Definitions
#define BUFFER_SIZE 16

/* RFM12B INTERFACE */
#define SCK 7 // SPI clock
#define SDO 6 // SPI Data output (RFM12B side)
#define SDI 5 // SPI Data input (RFM12B side)
#define CS 3 // SPI SS (chip select)
#define NIRQ 2 //
/* Macros */
#define HI_C(x) PORTC_OUT |= (1<<(x))
#define LO_C(x) PORTC_OUT &= ~(1<<(x))
#define HI_D(x) PORTD_OUT |= (1<<(x))
#define LO_D(x) PORTD_OUT &= ~(1<<(x))
#define WAIT_NIRQ_LOW() while(PORTD_IN&(1<<NIRQ))







//----------------------------------------------------------------------
//                      Public variables & functions                             
//----------------------------------------------------------------------


uint8_t rx_buffer[BUFFER_SIZE];




void remote_init(void);
/*
	initialization of RFM12B
	--
*/


uint8_t remote_recieve_wait(void);
/*
	wait until data byte is received
	--
*/


uint8_t remote_recieve_nowait(void);
/*
	return 0 [no data], >0 [some data are stored in buffer]
	for reading from buffer, use remote_read_buffer function
*/


void remote_read_buffer(uint8_t *destination);
/*
	copy data from buffer to destination
	--
*/




//************************************************************************/
//*                             EXAMPLES                                 */
//************************************************************************/
/*
	Example 1: check for data continuously
	
	include "remote.h"
	.
	.
	.
	int main(void)
	{
		remote_init();
		.
		.
		.

		while(1)
		{
			.
			.
			.

			if (remote_recieve_nowait())   // checks for data continuously
			{
				for(uint8_t x=0; x<BUFFER_SIZE; x++)
				{
					if (rx_buffer[x]==67)
					{
						// do the stuff
					}
				}
			}


		}
	}

	---------------------------------------
	Example 2: wait until some data arrives

	include "remote.h"
	.
	.
	.
	int main(void)
	{
		remote_init();
		.
		.
		.

		while(1)
		{
			.
			.
			.

			if (remote_recieve_wait()==67) //stops here and waits for first data byte
			{
				// do the stuff				
			}
		}
	}


	

*/


/************************************************************************/
/*                               CODE                                   */
/************************************************************************/
unsigned int writeCmd(unsigned int cmd) {
	unsigned char i;
	unsigned int recv;
	recv = 0;
	LO_C(SCK);
	_delay_ms(1);
	LO_D(CS);
	_delay_ms(1);
	for(i=0; i<16; i++) {
		if(cmd&0x8000) HI_C(SDI); else LO_C(SDI);
		_delay_ms(1);
		HI_C(SCK);
		_delay_ms(1);
		recv<<=1;
		if( PORTC_IN&(1<<SDO) ) {
			recv|=0x0001;
		}
		LO_C(SCK);
		_delay_ms(1);
		cmd<<=1;
	}
	HI_D(CS);
	_delay_ms(1);
	return recv;
}

unsigned char rfRecv() {
	unsigned int data;
		data = writeCmd(0x0000);
		if ( (data&0x8000) ) {
			data = writeCmd(0xB000);
			return (data&0x00FF);
		}
	return 0;
}
void FIFOReset() {
	writeCmd(0xCA81);
	writeCmd(0xCA83);
}

void rfInit() {
	writeCmd(0x80E7); //EL,EF,868band,12.0pF
	writeCmd(0x8299); //er,!ebb,ET,ES,EX,!eb,!ew,DC (bug was here)
	writeCmd(0xA640); //freq select
	writeCmd(0xC647); //4.8kbps
	writeCmd(0x94A0); //VDI,FAST,134kHz,0dBm,-103dBm
	writeCmd(0xC2AC); //AL,!ml,DIG,DQD4
	writeCmd(0xCA81); //FIFO8,SYNC,!ff,DR (FIFO level = 8)
	writeCmd(0xCED4); //SYNC=2DD4;
	writeCmd(0xC483); //@PWR,NO RSTRIC,!st,!fi,OE,EN
	writeCmd(0x9850); //!mp,90kHz,MAX OUT
	writeCmd(0xCC17); //!OB1,!OB0, LPX,!ddy,DDIT,BW0
	writeCmd(0xE000); //NOT USE
	writeCmd(0xC800); //NOT USE
	writeCmd(0xC040); //1.66MHz,2.2V
}


void remote_init(void)
{
	PORTC_DIRSET = (1<<SDI) | (1<<SCK);
	PORTD_DIRSET = (1<<CS);
	HI_D(CS);
	HI_C(SDI);
	LO_C(SCK);

	rfInit();	
	FIFOReset();
}


uint8_t remote_recieve_wait(void)
{
		
	//wait until data byte is received
	WAIT_NIRQ_LOW();
	
	for (uint8_t i=0; i<BUFFER_SIZE; i++) {
		rx_buffer[i] = rfRecv();
	}
	FIFOReset();
	return rx_buffer[0];
}

uint8_t remote_recieve_nowait(void){
/*
return 0 [no data], >0 [some data are stored in buffer]
for reading from buffer, use remote_read_buffer function
*/
	if(!(PORTD_IN&(1<<NIRQ)))
	{
		//read data into buffer
		for (uint8_t i=0; i<BUFFER_SIZE;i++)
		{
			rx_buffer[i]=rfRecv();
		}
		FIFOReset();
		return 1;
	}
	return 0;
}

void remote_read_buffer(uint8_t *destination)
{
	memcpy(&destination,&rx_buffer,sizeof(rx_buffer));
}

#endif /* _REMOTE_H_ */